@@ -1,18 +1,20 @@
 import json
 import unicodedata
-import urllib.parse
+import yarl
 import bs4
 import jsonpath_ng as jsonpath
 
 
-def make_context(url, content_type, content):
-  """Choose the right context based on the content type."""
-  # Heuristic to detect JSON mime-type.
-  # [1] https://www.iana.org/assignments/media-types/media-types.xhtml
-  if 'json' in content_type:
-    return JsonContext(url, JsonContext.parse(url, content))
-  else:
-    return HtmlContext(url, HtmlContext.parse(url, content))
+class Context:
+  @staticmethod
+  def create(url, content_type, content):
+    """Choose the right context based on the content type."""
+    # Heuristic to detect JSON mime-type.
+    # [1] https://www.iana.org/assignments/media-types/media-types.xhtml
+    if 'json' in content_type:
+      return JsonContext(url, JsonContext.parse(url, content))
+    else:
+      return HtmlContext(url, HtmlContext.parse(url, content))
 
 
 class StringContext:
@@ -71,7 +73,7 @@ class HtmlContext:
     doc = bs4.BeautifulSoup(html_string, features='html5lib')
     # Making links absolute is required to allow following.
     for tag in doc.findAll('a', href=True):
-      tag['href'] = urllib.parse.urljoin(url, tag['href'])
+      tag['href'] = str(url.join(yarl.URL(tag['href'])))
     # Replacing <br> tags with \n, prevents text concatenating.
     for br in doc.findAll('br'):
       br.replace_with("\n")
